==> persevero-1.1.0.tar: src/persevero.erl <==
-module(persevero).
-compile([no_auto_import, nowarn_unused_vars, nowarn_unused_function, nowarn_nomatch]).

-export([all_errors/1, custom_backoff/2, no_backoff/0, constant_backoff/1, linear_backoff/2, exponential_backoff/2, apply_constant/2, apply_jitter/2, apply_multiplier/2, apply_cap/2, execute_with_options/6, execute/4]).
-export_type([error/1, retry_data/2, mode/0]).

-type error(KYZ) :: {retries_exhausted, list(KYZ)} |
    {time_exhausted, list(KYZ)} |
    {unallowed_error, KYZ}.

-type retry_data(KZA, KZB) :: {retry_data,
        {ok, KZA} | {error, error(KZB)},
        list(integer()),
        integer()}.

-type mode() :: {max_attempts, integer()} | {expiry, integer()}.

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 63).
-spec all_errors(any()) -> boolean().
all_errors(_) ->
    true.

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 68).
-spec custom_backoff(integer(), fun((integer()) -> integer())) -> gleam@yielder:yielder(integer()).
custom_backoff(Wait_time, Next_wait_time) ->
    gleam@yielder:iterate(Wait_time, Next_wait_time).

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 77).
-spec no_backoff() -> gleam@yielder:yielder(integer()).
no_backoff() ->
    gleam@yielder:repeat(0).

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 83).
-spec constant_backoff(integer()) -> gleam@yielder:yielder(integer()).
constant_backoff(Wait_time) ->
    gleam@yielder:iterate(Wait_time, fun gleam@function:identity/1).

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 89).
-spec linear_backoff(integer(), integer()) -> gleam@yielder:yielder(integer()).
linear_backoff(Wait_time, Step) ->
    gleam@yielder:iterate(
        Wait_time,
        fun(_capture) -> gleam@int:add(_capture, Step) end
    ).

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 96).
-spec exponential_backoff(integer(), integer()) -> gleam@yielder:yielder(integer()).
exponential_backoff(Wait_time, Factor) ->
    gleam@yielder:iterate(
        Wait_time,
        fun(_capture) -> gleam@int:multiply(_capture, Factor) end
    ).

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 112).
-spec apply_constant(gleam@yielder:yielder(integer()), integer()) -> gleam@yielder:yielder(integer()).
apply_constant(Wait_stream, Adjustment) ->
    _pipe = Wait_stream,
    gleam@yielder:map(
        _pipe,
        fun(_capture) -> gleam@int:add(_capture, Adjustment) end
    ).

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 104).
-spec apply_jitter(gleam@yielder:yielder(integer()), integer()) -> gleam@yielder:yielder(integer()).
apply_jitter(Wait_stream, Upper_bound) ->
    apply_constant(Wait_stream, gleam@int:random(Upper_bound) + 1).

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 120).
-spec apply_multiplier(gleam@yielder:yielder(integer()), integer()) -> gleam@yielder:yielder(integer()).
apply_multiplier(Wait_stream, Factor) ->
    _pipe = Wait_stream,
    gleam@yielder:map(
        _pipe,
        fun(_capture) -> gleam@int:multiply(_capture, Factor) end
    ).

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 128).
-spec apply_cap(gleam@yielder:yielder(integer()), integer()) -> gleam@yielder:yielder(integer()).
apply_cap(Wait_stream, Max_wait_time) ->
    _pipe = Wait_stream,
    gleam@yielder:map(
        _pipe,
        fun(_capture) -> gleam@int:min(_capture, Max_wait_time) end
    ).

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 207).
-spec yielder_prepend(gleam@yielder:yielder(LAL), LAL) -> gleam@yielder:yielder(LAL).
yielder_prepend(Yielder, Element) ->
    gleam@yielder:yield(Element, fun() -> Yielder end).

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 190).
-spec configure_wait_stream(gleam@yielder:yielder(integer()), mode()) -> gleam@yielder:yielder(integer()).
configure_wait_stream(Wait_stream, Mode) ->
    Wait_stream@1 = begin
        _pipe = Wait_stream,
        _pipe@1 = yielder_prepend(_pipe, 0),
        gleam@yielder:map(
            _pipe@1,
            fun(_capture) -> gleam@int:max(_capture, 0) end
        )
    end,
    case Mode of
        {max_attempts, Max_attempts} ->
            _pipe@2 = Wait_stream@1,
            gleam@yielder:take(_pipe@2, Max_attempts);

        {expiry, _} ->
            Wait_stream@1
    end.

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 212).
-spec do_execute(
    gleam@yielder:yielder(integer()),
    fun((LAP) -> boolean()),
    mode(),
    fun((integer()) -> {ok, LAQ} | {error, LAP}),
    fun((integer()) -> nil),
    list(integer()),
    bigben@clock:clock(),
    list(LAP),
    integer(),
    birl:time(),
    integer()
) -> retry_data(LAQ, LAP).
do_execute(
    Wait_stream,
    Allow,
    Mode,
    Operation,
    Wait_function,
    Wait_time_acc,
    Clock,
    Errors_acc,
    Attempt_number,
    Start_time,
    Duration
) ->
    Should_execute = case Mode of
        {max_attempts, Max_attempts} ->
            Max_attempts > 0;

        {expiry, Expiry} ->
            (Expiry > 0) andalso (Duration < Expiry)
    end,
    case {Should_execute,
        begin
            _pipe = Wait_stream,
            gleam@yielder:step(_pipe)
        end} of
        {true, {next, Wait_time, Wait_stream@1}} ->
            Wait_function(Wait_time),
            Wait_time_acc@1 = [Wait_time | Wait_time_acc],
            Duration@1 = begin
                _pipe@1 = Clock,
                _pipe@2 = bigben@clock:now(_pipe@1),
                _pipe@3 = birl:difference(_pipe@2, Start_time),
                birl@duration:blur_to(_pipe@3, milli_second)
            end,
            case Operation(Attempt_number) of
                {ok, Result} ->
                    {retry_data,
                        {ok, Result},
                        begin
                            _pipe@4 = Wait_time_acc@1,
                            lists:reverse(_pipe@4)
                        end,
                        Duration@1};

                {error, Error} ->
                    case Allow(Error) of
                        true ->
                            do_execute(
                                Wait_stream@1,
                                Allow,
                                Mode,
                                Operation,
                                Wait_function,
                                Wait_time_acc@1,
                                Clock,
                                [Error | Errors_acc],
                                Attempt_number + 1,
                                Start_time,
                                Duration@1
                            );

                        false ->
                            {retry_data,
                                {error, {unallowed_error, Error}},
                                begin
                                    _pipe@5 = Wait_time_acc@1,
                                    lists:reverse(_pipe@5)
                                end,
                                Duration@1}
                    end
            end;

        {_, _} ->
            Error@1 = case Mode of
                {max_attempts, _} ->
                    {retries_exhausted,
                        begin
                            _pipe@6 = Errors_acc,
                            lists:reverse(_pipe@6)
                        end};

                {expiry, _} ->
                    {time_exhausted,
                        begin
                            _pipe@7 = Errors_acc,
                            lists:reverse(_pipe@7)
                        end}
            end,
            {retry_data,
                {error, Error@1},
                begin
                    _pipe@8 = Wait_time_acc,
                    lists:reverse(_pipe@8)
                end,
                Duration}
    end.

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 167).
-spec execute_with_options(
    gleam@yielder:yielder(integer()),
    fun((LAD) -> boolean()),
    mode(),
    fun((integer()) -> {ok, LAE} | {error, LAD}),
    fun((integer()) -> nil),
    bigben@clock:clock()
) -> retry_data(LAE, LAD).
execute_with_options(Wait_stream, Allow, Mode, Operation, Wait_function, Clock) ->
    do_execute(
        begin
            _pipe = Wait_stream,
            configure_wait_stream(_pipe, Mode)
        end,
        Allow,
        Mode,
        Operation,
        Wait_function,
        [],
        Clock,
        [],
        0,
        begin
            _pipe@1 = Clock,
            bigben@clock:now(_pipe@1)
        end,
        0
    ).

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 150).
-spec execute(
    gleam@yielder:yielder(integer()),
    fun((KZW) -> boolean()),
    mode(),
    fun(() -> {ok, KZX} | {error, KZW})
) -> {ok, KZX} | {error, error(KZW)}.
execute(Wait_stream, Allow, Mode, Operation) ->
    erlang:element(
        2,
        execute_with_options(
            Wait_stream,
            Allow,
            Mode,
            fun(_) -> Operation() end,
            fun gleam_erlang_ffi:sleep/1,
            bigben@clock:new()
        )
    ).

==> persevero-1.1.1.tar: src/persevero.erl <==
-module(persevero).
-compile([no_auto_import, nowarn_unused_vars, nowarn_unused_function, nowarn_nomatch]).

-export([all_errors/1, custom_backoff/2, no_backoff/0, constant_backoff/1, linear_backoff/2, exponential_backoff/2, apply_constant/2, apply_jitter/2, apply_multiplier/2, apply_cap/2, execute_with_options/6, execute/4]).
-export_type([error/1, retry_data/2, mode/0]).

-type error(KYU) :: {retries_exhausted, list(KYU)} |
    {time_exhausted, list(KYU)} |
    {unallowed_error, KYU}.

-type retry_data(KYV, KYW) :: {retry_data,
        {ok, KYV} | {error, error(KYW)},
        list(integer()),
        integer()}.

-type mode() :: {max_attempts, integer()} | {expiry, integer()}.

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 63).
-spec all_errors(any()) -> boolean().
all_errors(_) ->
    true.

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 68).
-spec custom_backoff(integer(), fun((integer()) -> integer())) -> gleam@yielder:yielder(integer()).
custom_backoff(Wait_time, Next_wait_time) ->
    gleam@yielder:iterate(Wait_time, Next_wait_time).

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 77).
-spec no_backoff() -> gleam@yielder:yielder(integer()).
no_backoff() ->
    gleam@yielder:repeat(0).

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 83).
-spec constant_backoff(integer()) -> gleam@yielder:yielder(integer()).
constant_backoff(Wait_time) ->
    gleam@yielder:iterate(Wait_time, fun gleam@function:identity/1).

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 89).
-spec linear_backoff(integer(), integer()) -> gleam@yielder:yielder(integer()).
linear_backoff(Wait_time, Step) ->
    gleam@yielder:iterate(
        Wait_time,
        fun(_capture) -> gleam@int:add(_capture, Step) end
    ).

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 96).
-spec exponential_backoff(integer(), integer()) -> gleam@yielder:yielder(integer()).
exponential_backoff(Wait_time, Factor) ->
    gleam@yielder:iterate(
        Wait_time,
        fun(_capture) -> gleam@int:multiply(_capture, Factor) end
    ).

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 112).
-spec apply_constant(gleam@yielder:yielder(integer()), integer()) -> gleam@yielder:yielder(integer()).
apply_constant(Wait_stream, Adjustment) ->
    _pipe = Wait_stream,
    gleam@yielder:map(
        _pipe,
        fun(_capture) -> gleam@int:add(_capture, Adjustment) end
    ).

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 104).
-spec apply_jitter(gleam@yielder:yielder(integer()), integer()) -> gleam@yielder:yielder(integer()).
apply_jitter(Wait_stream, Upper_bound) ->
    apply_constant(Wait_stream, gleam@int:random(Upper_bound) + 1).

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 120).
-spec apply_multiplier(gleam@yielder:yielder(integer()), integer()) -> gleam@yielder:yielder(integer()).
apply_multiplier(Wait_stream, Factor) ->
    _pipe = Wait_stream,
    gleam@yielder:map(
        _pipe,
        fun(_capture) -> gleam@int:multiply(_capture, Factor) end
    ).

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 128).
-spec apply_cap(gleam@yielder:yielder(integer()), integer()) -> gleam@yielder:yielder(integer()).
apply_cap(Wait_stream, Max_wait_time) ->
    _pipe = Wait_stream,
    gleam@yielder:map(
        _pipe,
        fun(_capture) -> gleam@int:min(_capture, Max_wait_time) end
    ).

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 198).
-spec configure_wait_stream(gleam@yielder:yielder(integer()), mode()) -> gleam@yielder:yielder(integer()).
configure_wait_stream(Wait_stream, Mode) ->
    Wait_stream@1 = begin
        _pipe = Wait_stream,
        _pipe@1 = gleam@yielder:prepend(_pipe, 0),
        gleam@yielder:map(
            _pipe@1,
            fun(_capture) -> gleam@int:max(_capture, 0) end
        )
    end,
    case Mode of
        {max_attempts, Max_attempts} ->
            _pipe@2 = Wait_stream@1,
            gleam@yielder:take(_pipe@2, Max_attempts);

        {expiry, _} ->
            Wait_stream@1
    end.

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 213).
-spec do_execute(
    gleam@yielder:yielder(integer()),
    fun((LAH) -> boolean()),
    mode(),
    fun((integer()) -> {ok, LAI} | {error, LAH}),
    fun((integer()) -> nil),
    list(integer()),
    bigben@clock:clock(),
    list(LAH),
    integer(),
    birl:time(),
    integer()
) -> retry_data(LAI, LAH).
do_execute(
    Wait_stream,
    Allow,
    Mode,
    Operation,
    Wait_function,
    Wait_time_acc,
    Clock,
    Errors_acc,
    Attempt_number,
    Start_time,
    Duration
) ->
    Should_execute = case Mode of
        {max_attempts, Max_attempts} ->
            Max_attempts > 0;

        {expiry, Expiry} ->
            (Expiry > 0) andalso (Duration < Expiry)
    end,
    case {Should_execute,
        begin
            _pipe = Wait_stream,
            gleam@yielder:step(_pipe)
        end} of
        {true, {next, Wait_time, Wait_stream@1}} ->
            Wait_function(Wait_time),
            Wait_time_acc@1 = [Wait_time | Wait_time_acc],
            Duration@1 = begin
                _pipe@1 = Clock,
                _pipe@2 = bigben@clock:now(_pipe@1),
                _pipe@3 = birl:difference(_pipe@2, Start_time),
                birl@duration:blur_to(_pipe@3, milli_second)
            end,
            case Operation(Attempt_number) of
                {ok, Result} ->
                    {retry_data,
                        {ok, Result},
                        begin
                            _pipe@4 = Wait_time_acc@1,
                            lists:reverse(_pipe@4)
                        end,
                        Duration@1};

                {error, Error} ->
                    case Allow(Error) of
                        true ->
                            do_execute(
                                Wait_stream@1,
                                Allow,
                                Mode,
                                Operation,
                                Wait_function,
                                Wait_time_acc@1,
                                Clock,
                                [Error | Errors_acc],
                                Attempt_number + 1,
                                Start_time,
                                Duration@1
                            );

                        false ->
                            {retry_data,
                                {error, {unallowed_error, Error}},
                                begin
                                    _pipe@5 = Wait_time_acc@1,
                                    lists:reverse(_pipe@5)
                                end,
                                Duration@1}
                    end
            end;

        {_, _} ->
            Error@1 = case Mode of
                {max_attempts, _} ->
                    {retries_exhausted,
                        begin
                            _pipe@6 = Errors_acc,
                            lists:reverse(_pipe@6)
                        end};

                {expiry, _} ->
                    {time_exhausted,
                        begin
                            _pipe@7 = Errors_acc,
                            lists:reverse(_pipe@7)
                        end}
            end,
            {retry_data,
                {error, Error@1},
                begin
                    _pipe@8 = Wait_time_acc,
                    lists:reverse(_pipe@8)
                end,
                Duration}
    end.

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 175).
-spec execute_with_options(
    gleam@yielder:yielder(integer()),
    fun((KZY) -> boolean()),
    mode(),
    fun((integer()) -> {ok, KZZ} | {error, KZY}),
    fun((integer()) -> nil),
    bigben@clock:clock()
) -> retry_data(KZZ, KZY).
execute_with_options(Wait_stream, Allow, Mode, Operation, Wait_function, Clock) ->
    do_execute(
        begin
            _pipe = Wait_stream,
            configure_wait_stream(_pipe, Mode)
        end,
        Allow,
        Mode,
        Operation,
        Wait_function,
        [],
        Clock,
        [],
        0,
        begin
            _pipe@1 = Clock,
            bigben@clock:now(_pipe@1)
        end,
        0
    ).

-file("/Users/josephlyons/Projects/Personal/gleam/persevero/src/persevero.gleam", 158).
-spec execute(
    gleam@yielder:yielder(integer()),
    fun((KZR) -> boolean()),
    mode(),
    fun(() -> {ok, KZS} | {error, KZR})
) -> {ok, KZS} | {error, error(KZR)}.
execute(Wait_stream, Allow, Mode, Operation) ->
    erlang:element(
        2,
        execute_with_options(
            Wait_stream,
            Allow,
            Mode,
            fun(_) -> Operation() end,
            fun gleam_erlang_ffi:sleep/1,
            bigben@clock:new()
        )
    ).
